#pragma once
#include <random>

#include <sol/sol.hpp>
#include <spdlog/spdlog.h>

namespace sva {
	class Sorter
	{
	public:
		struct SortResult
		{
			bool is_sorted;
			double elapsed_time;
			struct
			{
				size_t nSwap;
				size_t nSize;
				size_t nGreater;
				size_t nLesser;
				size_t nEqual;
			} calls;
		};

	private:
		std::vector<size_t> m_SortArray;

		SortResult* m_ActiveSortResult;
	public:
		Sorter(size_t size);

		SortResult run(const sol::protected_function& sort_func);

		static void setup(sol::state& state);

	protected:
		void list_swap(size_t, size_t);
		bool list_greater(size_t, size_t) const;
		bool list_less(size_t, size_t) const;
		bool list_equal(size_t, size_t) const;
		size_t list_size() const;

		bool is_sorted() const;

		void populate(size_t);
	};
}
